
/* ====================================================================
   Copyright 2002-2004   Apache Software Foundation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */
        


package org.apache.poi.hslf.extractor;

import java.io.*;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.apache.poi.hslf.model.*;

/**
 * This class can be used to extract text from a PowerPoint file.
 *  Can optionally also get the notes from one.
 *
 * @author Nick Burch
 */

public class PowerPointExtractor
{
  private SlideShow _show;
  private Sheet[] _sheets;
  private MetaSheet[] _msheets;

  /**
   * Basic extractor. Returns all the text, and optionally all the notes
   */
  public static void main(String args[]) throws IOException
  {
	if(args.length < 1) {
		System.err.println("Useage:");
		System.err.println("\tPowerPointExtractor [-notes] <file>");
		System.exit(1);
	}

	boolean notes = false;
	String file;
	if(args.length > 1) {
		notes = true;
		file = args[1];
	} else {
		file = args[0];
	}

	PowerPointExtractor ppe = new PowerPointExtractor(file);
	System.out.println(ppe.getText(true,notes));
	ppe.close();
  }

  /**
   * Creates a PowerPointExtractor
   * @param fileName
   */
  public PowerPointExtractor(String fileName) throws IOException {
	_show = new SlideShow(fileName);
	_sheets = _show.getSheets();
	_msheets = _show.getMetaSheets();
  }

  /**
   * Creates a PowerPointExtractor
   * @param iStream
   */
  public PowerPointExtractor(InputStream iStream) throws IOException {
	_show = new SlideShow(iStream);
	_sheets = _show.getSheets();
	_msheets = _show.getMetaSheets();
  }

  /**
   * Creates a PowerPointExtractor
   * @param fs
   */
  public PowerPointExtractor(POIFSFileSystem fs) throws IOException {
	_show = new SlideShow(fs);
	_sheets = _show.getSheets();
	_msheets = _show.getMetaSheets();
  }


  /**
   * Shuts down the underlying streams
   */
  public void close() throws IOException {
	_show.close();
	_show = null;
	_sheets = null;
	_msheets = null;
  }


  /**
   * Fetches all the slide text from the slideshow, but not the notes
   */
  public String getText() {
	return getText(true,false);
  }

  /**
   * Fetches all the notes text from the slideshow, but not the slide text
   */
  public String getNotes() {
	return getText(false,true);
  }

  /**
   * Fetches text from the slideshow, be it slide text or note text
   * @param getSlideText fetch slide text
   * @param getNoteText fetch note text
   */
  public String getText(boolean getSlideText, boolean getNoteText) {
	StringBuffer ret = new StringBuffer(); 

	if(getSlideText) {
		for(int i=0; i<_sheets.length; i++) {
			Sheet sheet = _sheets[i];
			TextRun[] runs = sheet.getTextRuns();
			for(int j=0; j<runs.length; j++) {
				TextRun run = runs[j];
				String text = run.getText();
				ret.append(text);
				if(! text.endsWith("\n")) {
					ret.append("\n");
				}
			}
		}
		if(getNoteText) {
			ret.append(" ");
		}
	}

	if(getNoteText) {
		for(int i=0; i<_msheets.length; i++) {
			MetaSheet msheet = _msheets[i];
			TextRun[] runs = msheet.getTextRuns();
			if(runs != null && runs.length > 0) {
				for(int j=0; j<runs.length; j++) {
					TextRun run = runs[j];
					String text = run.getText();
					ret.append(text);
					if(! text.endsWith("\n")) {
						ret.append("\n");
					}
				}
			}
		}
	}

	return ret.toString();
  }
}
