
/* ====================================================================
   Copyright 2002-2004   Apache Software Foundation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */
        


package org.apache.poi.hslf.model;

import java.util.*;
import java.io.*;

import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.apache.poi.poifs.filesystem.POIFSDocument;
import org.apache.poi.poifs.filesystem.DocumentEntry;

import org.apache.poi.util.LittleEndian;

import org.apache.poi.hslf.record.*;

/**
 * This class contains the main functionality for the Powerpoint file 
 * "reader". It is only a very basic class for now
 *
 * TODO:
 *  - figure out where the sheets and metasheets live from information
 *    in the file, rather than by scanning through the data
 *  - correctly figure out what character encoding is used for non unicode
 *    text runs, rather than relying on the system default
 *  - figure out how to match notes to their correct sheet
 * 
 * @author Nick Burch
 */

public class SlideShow
{
  private InputStream istream;
  private POIFSFileSystem filesystem;

  private byte[] _docstream;

  private Sheet[] _sheets;
  private MetaSheet[] _msheets;

  /**
   *  right now this function takes one parameter: a ppt file, and outputs
   *  the text it can find for it
   */
  public static void main(String args[]) throws IOException
  {
	SlideShow foo = new SlideShow(args[0]);

	Sheet[] sheets = foo.getSheets();
	for(int i=0; i<sheets.length; i++) {
		Sheet sheet = sheets[i];
		System.out.println("*Sheet " + sheet.getSheetNumber() + ":");
		TextRun[] runs = sheet.getTextRuns();
		for(int j=0; j<runs.length; j++) {
			TextRun run = runs[j];
			System.out.println("  * Text run " + run.getRunType());
			System.out.println("\n" + run.getText() + "\n");
		}
	}
  }

  /**
   * Constructs a Powerpoint document from fileName. Parses the document 
   * and places all the important stuff into data structures.
   *
   * @param fileName The name of the file to read.
   * @throws IOException if there is a problem while parsing the document.
   */
  public SlideShow(String fileName) throws IOException
  {
  	this(new FileInputStream(fileName));
  }
  
  /**
   * Constructs a Powerpoint document from an input stream. Parses the 
   * document and places all the important stuff into data structures.
   *
   * @param inputStream the source of the data
   * @throws IOException if there is a problem while parsing the document.
   */
  public SlideShow(InputStream inputStream) throws IOException
  {
        //do Ole stuff
		this(new POIFSFileSystem(inputStream));
        istream = inputStream;
  }

  /**
   * Constructs a Powerpoint document from a POIFS Filesystem. Parses the 
   * document and places all the important stuff into data structures.
   *
   * @param filesystem the POIFS FileSystem to read from
   * @throws IOException if there is a problem while parsing the document.
   */
  public SlideShow(POIFSFileSystem filesystem) throws IOException
  {
		this.filesystem = filesystem;

        // Go find a PowerPoint document in the stream
        // Save anything useful we come across
        readFIB();
  }


  /**
   * Shuts things down. Closes underlying streams etc
   *
   * @throws IOException
   */
  public void close() throws IOException
  {
	if(istream != null) {
		istream.close();
	}
	filesystem = null;
  }


  /**
   * Extracts the main document stream from the POI file then hands off 
   * to other functions that parse other areas.
   *
   * @throws IOException
   */
  private void readFIB() throws IOException
  {
	// Get the main document stream
	DocumentEntry docProps =
		(DocumentEntry)filesystem.getRoot().getEntry("PowerPoint Document");

	// Grab the document stream
	_docstream = new byte[docProps.getSize()];
	filesystem.createDocumentInputStream("PowerPoint Document").read(_docstream);

	// The format of records in a powerpoint file are:
	//   <little endian 2 byte "info">
	//   <little endian 2 byte "type">
	//   <little endian 4 byte "length">
	// If it has a zero length, following it will be another record
	//		<xx xx yy yy 00 00 00 00> <xx xx yy yy zz zz zz zz>
	// If it has a length, depending on its type it may have children or data
	// If it has children, these will follow straight away
	//		<xx xx yy yy zz zz zz zz <xx xx yy yy zz zz zz zz>>
	// If it has data, this will come straigh after, and run for the length
	//      <xx xx yy yy zz zz zz zz dd dd dd dd dd dd dd>
	// All lengths given exclude the 8 byte record header
	// (Data records are known as Atoms)

	// Document should start with:
	//   0F 00 E8 03 ## ## ## ##
    //     (type 1000 = document, info 00 0f is normal, rest is document length)
	//   01 00 E9 03 28 00 00 00
	//     (type 1001 = document atom, info 00 01 normal, 28 bytes long)
	//   80 16 00 00 E0 10 00 00 xx xx xx xx xx xx xx xx
	//   05 00 00 00 0A 00 00 00 xx xx xx
	//     (the contents of the document atom, not sure what it means yet)
	//   (records then follow)

	// When parsing a document, look to see if you know about that type
	//  of the current record. If you know it's a type that has children, 
	//  process the record's data area looking for more records
	// If you know about the type and it doesn't have children, either do
	//  something with the data (eg TextRun) or skip over it
	// If you don't know about the type, play safe and skip over it (using
	//  its length to know where the next record will start)
	//
	// For now, this work is handled by Record.findChildRecords

	Record[] records = Record.findChildRecords(_docstream,0,_docstream.length);

	// For holding the sheets
	Vector sheets = new Vector(10);
	// For holding the meta sheets
	Vector metaSheets = new Vector(10);

	// Look for PPDrawings
	for(int i=0; i<records.length; i++) {
		// Look in Notes
		if(records[i].getRecordType() == 1008L) {
			Record[] children = records[i].getChildRecords();
			for(int j=0; j<children.length; j++) {
				if(children[j].getRecordType() == 1036L) {
					PPDrawing ppd = (PPDrawing)children[j];
					MetaSheet ms = ppd.getMetaSheet();
					if(ms != null) {
						metaSheets.add(ms);
						//System.out.println("Found a meta sheet at " + i + " " + j);
					} else {
						System.err.println("PPDrawing at " + i + ", " + j + " didn't have a meta sheet!");
					}
				}
			}
		}
		// Look in Document -> SlideListWithText
		if(records[i].getRecordType() == 1000L) {
			Record[] children = records[i].getChildRecords();
			for(int j=0; j<children.length; j++) {
				if(children[j].getRecordType() == 4080L) {
					SlideListWithText slwt = (SlideListWithText)children[j];
					Sheet[] s = slwt.getSheets();
					//System.out.println("Found " + s.length + " sheets at " + i + " " + j);
					for(int k=0; k<s.length; k++) {
						sheets.add(s[k]);
					}
				}
			}
		}
	}


	// Put the vectors into arrays
	_sheets = new Sheet[sheets.size()];
	_msheets = new MetaSheet[metaSheets.size()];
	for(int i=0; i<_sheets.length; i++) {
		_sheets[i] = (Sheet)sheets.get(i);
	}
	for(int i=0; i<_msheets.length; i++) {
		_msheets[i] = (MetaSheet)metaSheets.get(i);
	}
  }


  // Accesser methods follow

  /**
   * Returns an array of all the normal Sheets found in the slideshow
   */
  public Sheet[] getSheets() { return _sheets; }

  /**
   * Returns an array of all the meta Sheets found in the slideshow
   */
  public MetaSheet[] getMetaSheets() { return _msheets; }
}
