
/* ====================================================================
   Copyright 2002-2004   Apache Software Foundation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */
        


package org.apache.poi.hslf.model;

import java.util.*;
import java.io.*;

import org.apache.poi.util.LittleEndian;

/**
 * This class represents a run of text in a powerpoint document. That
 *  run could be text on a sheet, or text in a note.
 *  It is only a very basic class for now
 *
 * @author Nick Burch
 */

public class TextRun
{
  private byte[] _run;
  private boolean _isUnicode;
  private String _rawText;
  private String _text;
  private byte _runType;

  /**
   * Constructs a Text Run from an underlying sheet or meta sheet.
   * Extracts the text into a string, handling unicode stuff if needed
   *
   * @param bytes the byte array to get data from
   * @param start the point in the byte array to start at
   * @param length how far into the byte array to read
   */
  public TextRun(byte[] bytes, int start, int length)
  {
	_run = new byte[length];
	System.arraycopy(bytes,start,_run,0,length);

	// Zip forward, and see what it is:
	//  00 00 9f 0f 04 00 00 00 ## 00 00 00 
	//  00 00 [A0 | A8] 0f (len len) 00 00 text text
	//
	// the contents of ## dictate what sort of text it is,
	//  eg body, notes, centre title etc. 
	//  See TextHeaderAtom (3999) for full details
	// If it's A0 0F then that's 4000 -> TextCharsAtom -> unicode
	// If it's A8 0F then that's 4008 -> TextBytesAtom -> not unicode
	//  (still need to figure out exactly which character set this is)
	_runType = _run[7];
	if(_run[13] == (0xa0 - 256)) {
		// unicode (character) text
		_isUnicode = true;
	} else if(_run[13] == (0xa8 - 256)) {
		// byte text
		_isUnicode = false;
	} else {
		// who knows
		System.err.println("Hit unknown text text, run type is " + _runType + ", text type is " + _run[13]);
		_isUnicode = false;
	}

	// Figure out how long the text is
	long size = LittleEndian.getUInt(_run,15);

	// Create the text from the bytes
	if(_isUnicode) {
		try {
			//System.out.println("Found unicode text");
			_rawText = pptUnicodeMakeString(_run,19,(int)size);
		} catch(UnsupportedEncodingException uee) {
			throw new Error("Hit unicode text, but utf-16 support not present:" + uee);
		}
	} else {
		_rawText = new String(_run,19,(int)size);
	}

	// PowerPoint seems to store files with \r as the line break
	// The messes things up on everything but a Mac, so translate
	//  them to \n
	_text = _rawText.replace('\r','\n');
  }


  /**
   * Constructs a String from a unicode based length of bytes. Java
   *  assumes the format for UTF-16 is (high)(low), but PPT stores it
   *  as (low)(high), yey...
   *
   * @param b the byte array to get data from
   * @param start the point in the byte array to start at
   * @param len how far into the byte array to read
   */
  public static String pptUnicodeMakeString(byte[] b, int start, int len) throws UnsupportedEncodingException {
	byte[] reversed = new byte[len];
	for(int i=0; i<len; i+= 2) {
		reversed[i] = b[i+start+1];
		reversed[i+1] = b[i+start];
	}
	return new String(reversed,"UTF-16");
  }


  // Accesser methods follow

  /**
   * Returns the text content of the run
   */
  public String getText() { return _text; }

  /**
   * Returns the raw text content of the run. This hasn't had any
   *  changes applied to it, and so is probably unlikely to print
   *  out nicely.
   */
  public String getRawText() { return _rawText; }

  /**
   * Returns the raw type of the run, from offset 7
   * (This corresponds to the 4 byte type from a TextHeaderAtom)
   */
  public byte getRunType() { return _runType; }

} 
