
/* ====================================================================
   Copyright 2002-2004   Apache Software Foundation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */
        

package org.apache.poi.hslf.record;

import org.apache.poi.util.LittleEndian;
import org.apache.poi.hslf.model.MetaSheet;

/**
 * These are tricky beasts. They contain the text of notes, and
 *  of master slides
 * For now, we scan through looking for interesting bits, then creating
 *  the helpful MetaSheet from model for them
 *
 * @author Nick Burch
 */

public class PPDrawing extends Record
{
	private byte[] _contents;
	private long _type;

	private MetaSheet metaSheet;

	/** 
	 * Create a new holder for slide records
	 */
	protected PPDrawing(byte[] source, int start, int len) {
		_contents = new byte[len];
		System.arraycopy(source,start,_contents,0,len);
		_type = LittleEndian.getUShort(_contents,2);

		// Find our children
		findSheets();
	}

	private void findSheets() {
		// ** At the moment, still scanning for sheets **

		// Loop over the data of this PPDrawing, looking for a 
		// meta sheets
		for(int i=0; i < (_contents.length - 28); i++) {
			// Look for new pages
			// These look like:
			//     00 f3 03 14 00 00 00 ##
			// (where ## is the page number, as a byte)
			if(_contents[i] == 0x00 &&
				_contents[i+1] == (0xf3-256) &&
				_contents[i+2] == 0x03 &&
				_contents[i+3] == 0x14 &&
				_contents[i+4] == 0x00 &&
				_contents[i+5] == 0x00 &&
				_contents[i+6] == 0x00 
			) {
				// Sheets seem to contain:
				// 28 bytes from i:
				//   00 9f 0f 04 00 00 00 ## 00 00 00 00 00 
				//   a0 0f (len len) 00 00 (text text text)
				// some crap
				//   00 9f 0f 04 00 00 00 ## 00 00 00 00 00
				//   a0 0f (len len) 00 00 (more text text)
				//
				// First ## is 00 (page title), rest can be any number

				// Have yet to spot what defines if unicode text
				//  (xx 00 xx 00) or normal text (xx xx xx)

				System.err.println("");
				System.err.println("*** Found a normal sheet inside a PPDrawing, that isn't supposed to happen! ***");
				System.err.println("");
			}

			// Look for meta page entries
			// These look like:
			//     00 c3 0b 08 00 00 00 tt 00 00 00 ww xx yy 0z 0f 00 0d f0 ##
			// tt is type:
			//			00 = master page, notes pages type b
			//			01 = notes page type c or 2nd part of master page 
			//			     or subtitle master page
			//			03 = handout master page
			//	ww xx is one of:
			//			0b 00 = notes page type b
			//			0c 00 = notes page type c
			//			01 00 = master page
			//			02 00 = 2nd part of master page
			//			04 02 = subtitle master page
			//			06 02 = handout page
			// yy is unknown
			// 0z is one of:
			//			00 for 2nd part of master page
			//			01 for notes page, master page, handout page
			//			05 for subtitle master page
			// ## is unknown, but might be a length
			//
			// Other spotted ones include
			//		00   0A 02  01
			//      01   07 00  01
			//      02   09 02  05
			//      03   08 02  05
			//      04   09 02  01
			//      05   08 02  01
			if(_contents[i] == 0x00 &&
				_contents[i+1] == (0xc3-256) &&
				_contents[i+2] == 0x0b &&
				_contents[i+3] == 0x08 &&
				_contents[i+4] == 0x00 &&
				_contents[i+5] == 0x00 &&
				_contents[i+6] == 0x00 
			) {
				//System.out.println("Found new meta sheet at " + i);

				metaSheet = new MetaSheet(_contents,i,_contents.length - i);
				i = _contents.length;
			}
		}
	}

	/**
	 * Get access to the underlying MetaSheet (if present, otherwise null)
	 */
	public MetaSheet getMetaSheet() { return metaSheet; }

	/** 
	 * For now, pretend to be an atom
	 */
	public boolean isAnAtom() { return true; }

	/**
	 * Return the value we were given at creation
	 */
	public long getRecordType() { return _type; }

	/** 
	 * We're pretending to be an atom, so return null
	 */
	public Record[] getChildRecords() { return null; }

	/**
	 * Fetch the byte representation of this record. Really the
	 *  only use of this record, so it can be written back to disk
	 */
	protected byte[] getByteRepresentation() { return _contents; }
}
