
/* ====================================================================
   Copyright 2002-2004   Apache Software Foundation

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */
        


package org.apache.poi.hslf.record;
import java.util.Vector;
import org.apache.poi.util.LittleEndian;


/**
 * This abstract class represents a record in the PowerPoint document.
 * All the record classes implement this.
 *
 * @author Nick Burch
 */

public abstract class Record
{
	/** 
	 * Is this record type an Atom record (only has data),
	 *  or is it a non-Atom record (has other records)?
	 */
	public abstract boolean isAnAtom();

	/**
	 * Returns the type (held as a little endian in bytes 3 and 4)
	 *  that this class handles
	 */
	public abstract long getRecordType();

	/** 
	 * Fetch all the child records of this record
	 * If this record is an atom, will return null
	 * If this record is a non-atom, but has no children, will return 
	 *  an empty array
	 */
	public abstract Record[] getChildRecords();

	/**
	 * Fetch the byte representation of this record, and its children
	 * Used when writing back out to disk
	 * (Normally, atom classes will keep their bytes around, but
	 *  non atom classes will just request the bytes from their 
	 *  children, then chuck on their header and return)
	 */
	protected abstract byte[] getByteRepresentation();

	/**
	 * Default method for finding child records of a given record
	 */
	public static Record[] findChildRecords(byte[] b, int start, int len) {
		Vector children = new Vector(5);

		// Jump our little way along, creating records as we go
		int pos = start;
		while(pos <= (len-8)) {
			long type = LittleEndian.getUShort(b,pos+2);
			long rlen = LittleEndian.getUInt(b,pos+4);
//System.out.println("Found a " + type + " at pos " + pos + " (" + Integer.toHexString(pos) + "), len " + rlen);
			Record r = createRecordForType(type,b,pos,8+(int)rlen);
			children.add(r);
			pos += 8;
			pos += rlen;
		}

		// Turn the vector into an array, and return
		Record[] cRecords = new Record[children.size()];
		for(int i=0; i < children.size(); i++) {
			cRecords[i] = (Record)children.get(i);
		}
		return cRecords;
	}

	/**
	 * For a given type (little endian bytes 3 and 4 in record header),
	 *  byte array, start position and length:
	 *  will return a Record object that will handle that record
	 */
	protected static Record createRecordForType(long type, byte[] b, int start, int len) {
		// Default is to use UnknownRecordPlaceholder
		// When you create classes for new Records, add them here
		switch((int)type) {
			// Document
			case 1000:
				return new DummyRecordWithChildren(b,start,len);
				
			// "Slide" (Sheet lives somewhere else....)
			case 1006:
				return new DummyRecordWithChildren(b,start,len);

			// "Notes" (MetaSheet lives inside this)
			case 1008:
				return new DummyRecordWithChildren(b,start,len);
				
			// MainMaster (MetaSheet lives inside the PPDrawing inside this)
			case 1016:
				return new DummyRecordWithChildren(b,start,len);

			// PPDrawing (MetaSheet lives inside this)
			case 1036:
				return new PPDrawing(b,start,len);

			// SlideListWithText (Many Sheets live inside here)
			case 4080:
				return new SlideListWithText(b,start,len);
				
			default:
				return new UnknownRecordPlaceholder(b,start,len);
		}
	}
}
